#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
* Filename: sdk.py
* Description:
* Time: 2024.12.05
*/
"""
import base64

import requests


class ZoomEyeDict:

    def __init__(self, data):
        self.dict = dict(data)

    def find(self, key):
        """
        get the value in the nested dictionary by <"key1.key2.key3">
        :param key: str, dictionary key like: "a.b.c"
        :return:
        """
        values = None
        # is dict?
        if isinstance(self.dict, dict):
            keys = key.split(".")
            inputData = self.dict
            for k in keys:
                if k == 'geoinfo' and inputData.get(k) == None:
                    k = "aiweninfo" if inputData.get("aiweninfo") else "ipipinfo"
                if inputData.get(k) is not None:
                    values = inputData.get(k)
                else:
                    values = None
                if isinstance(values, list):
                    if len(values) != 0:
                        values = values[0]
                    else:
                        values = '[unknown]'
                inputData = values
            return values
        else:
            raise TypeError("the parameter you pass in must be a dictionary, not a {}".format(type(self.dict)))


class ZoomEye:

    def __init__(self, api_key=""):
        self.api_key = api_key

        # 3.0.0
        self.search_api = "https://api.zoomeye.ai/v2/search"
        self.userinfo_api = "https://api.zoomeye.ai/v2/userinfo"

    def _request(self, url, params=None, headers=None, method='GET'):
        """
        encapsulate the requests part
        :param url: send request url
        :param params: request params
        :param headers: request header
        :param method: send request method， only support method GET and POST
        :return: json data
        """
        # if method is "GET" use requests.get
        if method == "GET":
            resp = requests.get(url, params=params, headers=headers)
        # post json body
        elif method == "POST" and headers.get("Content-Type", "") == "application/json":
            resp = requests.post(url, json=params, headers=headers)
        # request method is "POST"
        else:
            resp = requests.post(url, data=params, headers=headers)
            # print(resp.text)
        # if response succeed and status code is 200 return json data
        if resp and resp.status_code == 200:
            data = resp.json()
            return data
        # Request data exceeds the total amount of ZoomEye data,
        # return all data instead of throwing an exception
        elif resp.status_code == 403 and 'specified resource' in resp.text:
            return None
        # if response succeed and status code is not 200 return error format json
        # others error return unknown error
        else:
            raise ValueError(resp.json().get('message'))

    def _check_header(self):
        """
        2023-04 remove username & password authenticate
        only support API-KEY authenticate
        """
        if self.api_key:
            headers = {
                'API-KEY': self.api_key,
            }
        else:
            headers = {}
        # add user agent
        headers[
            "User-Agent"] = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.0.0 Safari/537.36"
        return headers

    """
    version: 3.0.0
    """

    def userinfo(self):
        headers = self._check_header()
        resp = self._request(self.userinfo_api,
                             method='POST',
                             params={},
                             headers=headers)
        return resp

    def search(self, dork, qbase64='', page=1, pagesize=20, sub_type='all', fields='', facets='') -> dict:
        """
         get network asset information based on query conditions.
         mainly used to search asset data from zoomeye.
        please see: https://www.zoomeye.ai/doc/
        :param dork:str,
                    ex: country=cn
                    dork to search
        :param qbase64:str,
                    ex: country=cn
                    base64 dork to search
        :param sub_type:str,
                    specify the type of data to search
                    supported : 'v4', 'v6', 'web', 'all
        :param page: int,
                    specify the number of pages to return data, each page contains 20 data
        :param pagesize: int,
                    specify the number of pagesize to search
        :param fields: str,
                        display data based on input fields
        :param facets: str,
                     if this parameter is specified,
                     the corresponding data will be displayed
                     at the end of the returned result.
                     supported : 'product', 'device', 'service', 'os', 'port', 'country', 'subdivisions', 'city'
        :return: dict

        """
        if qbase64:
            dork = qbase64
        elif qbase64 == '' and dork != '':
            dork = base64.b64encode(dork.encode('utf-8')).decode('utf-8')
        else:
            raise Exception("dork and qbase64 can't be empty at the same time")
        headers = self._check_header()
        headers["Content-Type"] = "application/json"
        params = {"qbase64": dork, "sub_type": sub_type, "page": page, "facets": facets, "fields": fields,
                  "pagesize": pagesize}
        resp = self._request(self.search_api,
                             method='POST',
                             params=params,
                             headers=headers)
        return resp


def zoomeye_api_test():
    zoomeye = ZoomEye()
    zoomeye.api_key = input('ZoomEye API-KEY:')

    """
    version: 3.0.0
    """
    print(zoomeye.userinfo())

    data = zoomeye.search('country=cn')
    print(data)

if __name__ == "__main__":
    zoomeye_api_test()



"""
* Filename: show.py
* Description: used to print different data on the terminal.
* Time: 2024.12.05
*/
"""
import re

from colorama import init

from zoomeyeai import plotlib
from zoomeyeai.tools import convert_str

# solve the color display error under windows terminal
init(autoreset=True)


def printf(s, color="white"):
    """
    print result in terminal
    :param s: str, printed string
    :param color: str, printed string color
    :return:
    """
    colors = {
        "red": '\033[31m',  # red
        "green": '\033[32m',  # green
        "yellow": '\033[33m',  # yellow
        "blue": '\033[34m',  # blue
        "white": '\033[37m',  # white
    }

    # default color
    Default = '\033[0m'  # item default color
    print('{}{}{}'.format(colors.get(color, "WHITE"), s, Default))


def print_filter(keys, data_list, condition=None):
    """
    used to display user filtered data on the terminal
    :param keys: user input key, is str
    :param data_list: filter data ,is list
    :param condition: list,
    :return:
    """
    total = 0
    # print title
    title = ""
    for key in keys.split(","):
        title += "{:<30}".format(key)
    printf("{}".format(title), color="green")
    # print data
    for i in data_list:
        items = ""
        for j in i:
            if len(str(j)) != 0 and isinstance(j, list):
                if len(j) == 0:
                    j = '[unknown]'
                elif isinstance(j[0], str):
                    j = j[0]
                elif isinstance(j[0], dict):
                    j = j[0].get('name', '[unknown]')
                else:
                    j = j
            elif isinstance(j, dict):
                j = j.get('name', '[unknown]')
            else:
                if len(str(j)) == 0:
                    j = '[unknown]'
            j_hex = convert_str(str(j))
            # match to content highlight
            if condition:
                for item in condition:
                    k, v = item.split('=')
                    result_re = re.search(v, j_hex, re.I | re.M)
                    if result_re:
                        # replace to highlight
                        color_content = "\033[31m{}\033[0m".format(result_re.group())
                        j_hex = j_hex.replace(result_re.group(), color_content)
            items += "{:<30}".format(j_hex)
        total += 1
        printf(items)
    print()
    printf("total: {}".format(total))


def print_facets(facets, facet_data: dict, total, figure, table):
    """
    used to print data that users need to count
    :param facets: str, statistical data
    :param facet_data: dict, all facets data
    :param total:int, data total
    :param figure:int, data total
    {"app": [{"name": 'xx', 'count': xxx}{"name": 'xx', 'count': xxx}{"name": 'xx', 'count': xxx}]}
    :return:
    """
    if not facet_data:
        return
    # print facet data

    print(' ' + '-' * 40)
    printf(" ZoomEye total data:{}".format(total))
    for facet in facets.split(","):
        names = []
        counts = []
        pie_info = []
        facet_total = {}
        facet_count = 0

        if table.get(facet.strip()) is None:
            support_fields = ','.join(list(table.keys()))
            printf("facet command has unsupport fields [{}], support fields has [{}]".format(facet.strip(),
                                                                                             support_fields),
                   color='red')
            exit(0)
        print(' {:-^40}'.format(facet + " Top 10"))
        if figure is None:
            printf(" {:<35}{:<20}".format(facet, "count"), color="green")
        f = table.get(facet)
        for t in facet_data.get(f):
            facet_count += t.get('count')
        facet_total[facet] = facet_count

        for d in facet_data.get(f):
            name = d.get("name")
            if len(str(name)) == 0:
                name = "[unknown]"

            count = d.get("count")
            # get histogram data
            names.append(name)
            counts.append(count)
            # get pie chart data
            # three decimal places
            pie = (name, round(count / facet_total.get(facet), 3))
            pie_info.append(pie)
            if figure is None:
                printf(" {:<35}{:<20}".format(name, count))
        # pie chart
        if figure and figure == 'pie':
            plotlib.show_pie_chart(pie_info)
        # histogram
        if figure and figure == 'hist':
            plotlib.generate_histogram(counts, names, force_ascii=True)



"""
* Filename: core.py
* Description: cli core function, processing various requests
* Time: 2024.12.05
*/
"""

import os

from zoomeyeai import config, file, show
from zoomeyeai.sdk import ZoomEye

# save zoomeye config folder
zoomeye_dir = os.path.expanduser(config.ZOOMEYE_CONFIG_PATH)


def init_key(key):  # 初始化API密钥并将其保存在本地配置文件中

    """
    initialize through the api key, write the api key to the local configuration file,
    theoretically it will never expire unless you remake the api key
    :param key: user input API key
    :return:
    """
    file.check_exist(zoomeye_dir)
    key = key.strip()
    try:
        zoom = ZoomEye(api_key=key)
    except Exception:
        return
    # api key save path
    key_file = zoomeye_dir + "/apikey"
    # display the remaining resources of the current account
    user_data = zoom.userinfo()
    print(user_data)
    if user_data.get("code") == 60000:
        show.printf("Role: {}".format(user_data.get('data', {}).get('subscription', {}).get('plan', '')))
        show.printf("Points: {}".format(user_data.get('data', {}).get('subscription', {}).get("points", 0)))
        show.printf("Zoomeye Points: {}".format(user_data.get('data', {}).get('subscription', {}).get("zoomeye_points", 0)))
    # save api key
    with open(key_file, 'w') as f:
        f.write(key)
    show.printf("successfully initialized", color="green")
    # change the permission of the configuration file to read-only
    os.chmod(key_file, 0o600)


def init(args):  # 根据用户输入的参数来选择初始化方法，可以通过API密钥来进行初始化
    """
    the initialization processing function will select the initialization method according to the user's input.
    :param args:
    :return:
    """
    api_key = args.apikey
    # use api key init
    if api_key:
        init_key(api_key)
        return
    # invalid parameter
    show.printf("input parameter error!", color="red")
    show.printf("please run <zoomeye init -h> for help.", color="red")


def info(args):  # 打印当前用户的身份和本月剩余的数据配额
    """
    used to print the current identity of the user and the remaining data quota for the month
    :return:
    """
    api_key = file.get_auth_key()
    zm = ZoomEye(api_key=api_key)
    # get user information
    user_response = zm.userinfo()

    if user_response and user_response.get('code') == 60000:
        user_data = user_response.get('data')
        # show in the terminal
        show.printf("username: {}".format(user_data["username"]))
        show.printf("email: {}".format(user_data["email"]))
        show.printf("phone: {}".format(user_data["phone"]))
        show.printf("created_at: {}".format(user_data["created_at"]))
        show.printf("Subscription:: {}".format(user_data["subscription"]))


def search(args):
    dork = args.dork
    page = int(args.page)
    pagesize = int(args.pagesize)
    facets = args.facets
    fields = args.fields
    sub_type = args.sub_type
    figure = args.figure

    api_key = file.get_auth_key()
    zm = ZoomEye(api_key=api_key)
    data = zm.search(dork, page=page, pagesize=pagesize, facets=facets, fields=fields, sub_type=sub_type)

    data_list = data.get('data', [])
    facets_data = data.get('facets', {})
    total = data.get('total', {})

    if len(data_list) > 0:
        keys = data_list[0].keys()
        show.print_filter(",".join(keys), [[i.get(key) for key in keys] for i in data_list])

    if facets_data:
        show.print_facets(facets, facets_data, total, figure, {
            'type': 'type',
            'product': 'product',
            'device': 'device',
            'service': 'service',
            'os': 'os',
            'port': 'port',
            'subdivisions': 'subdivisions',
            'country': 'country',
            'city': 'city',
        })

    # show.printf("please run <zoomeye search -h> for help.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-


"""
* Filename: tools.py
* Description:
* Time: 2025.12.05
*/
"""
import hashlib

from zoomeyeai import config


def md5_convert(string):
    """
    calculate the md5 of a string
    :param string: input string
    :return: md5 string
    """
    m = hashlib.md5()
    m.update(string.encode('utf-8'))
    return m.hexdigest()

def convert_str(s):
    """
    convert arbitrary string to another string, for print and human readable
    :param:
    :return:
    """
    res = []
    d = {
        '\n': '\\n',
        '\r': '\\r',
        '\t': '\\t',
        '\b': '\\b',
        '\a': '\\a',
    }
    for c in s:
        if c in d.keys():
            res.append(d[c])
        else:
            res.append(c)
    return ''.join(res)

def omit_str(text, index=0):
    """
    ignore part of the string
    :param text: str, text to be omitted
    :param index: int, position omitted on the longest basis
    :return: str, ex: string...
    """
    if len(text) < config.STRING_MAX_LENGTH:
        return text

    return text[:config.STRING_MAX_LENGTH - index] + '...'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
* Filename: __init__.py
* Description:
* Time: 2024.12.05
*/
"""

__name__ = 'zoomeyeai'
__package__ = 'zoomeyeai'
__version__ = 'v3.0.0'
__site__ = "https://www.zoomeye.ai/doc"
